/*
 * EXCHANGE.C
 *
 * Main entry code for application to demonstrate simepl NetDDE
 * transactions.
 *
 * Copyright (c)1992 Microsoft Corporation, All Right Reserved
 *
 * Kraig Brockschmidt, Software Design Engineer
 * Microsoft Systems Developer Relations
 * One Microsoft Way
 * Redmond, WA  98052
 *
 * Internet  :  kraigb@microsoft.com
 * Compuserve:  70750,2344
 * Fax       :  (206)936-7329
 */

#define STRICT
#include <windows.h>
#include <ddeml.h>
#include "exchange.h"
#include "connect.h"



/*
 * WinMain
 *
 * Purpose:
 *  Main entry point of application.
 *
 * Parameters:
 *  Standard
 *
 * Return Value:
 *  Value to return to Windows--termination code.
 */

int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hInstPrev
                   , LPSTR pszCmdLine, int nCmdShow)
    {
    MSG         msg;
    BOOL        fSuccess=TRUE;
    PAPPVARS    pAV;

    //Attempt to allocate and initialize the application
    pAV=AppPAllocate(&fSuccess, hInst, hInstPrev, pszCmdLine, nCmdShow);

    if (NULL==pAV || !fSuccess)
        return (int)AppPFree(pAV);

    while (GetMessage(&msg, NULL, 0,0 ))
        {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
        }

    AppPFree(pAV);
    return msg.wParam;
    }





/*
 * ExchangeWndProc
 *
 * Purpose:
 *  Window class procedure.  Standard callback.
 *
 * Parameters:
 *  Standard
 *
 * Return Value:
 *  Standard
 */

LONG FAR PASCAL ExchangeWndProc(HWND hWnd, UINT iMsg
                            , WPARAM wParam, LPARAM lParam)
    {
    static int      cTimers;
    PAPPVARS        pAV;
    DLGPROC         pfn;
    HDC             hDC;
    PAINTSTRUCT     ps;
    int             cch, i;
    RECT            rc;
    HMENU           hMenu;
    WORD            wID;
    WORD            wCode;
    HWND            hWndMsg;


    hWndMsg=(HWND)(UINT)lParam;
    wID=LOWORD(wParam);
    #ifdef WIN32
        wCode=HIWORD(wParam);
    #else
        wCode=HIWORD(lParam);
    #endif

    //This will be valid for all messages except WM_NCCREATE
    pAV=(PAPPVARS)(UINT)GetWindowLong(hWnd, 0);


    switch (iMsg)
        {
        case WM_NCCREATE:
            //CreateWindow passed pAV to us.
            pAV=(PAPPVARS)((UINT)((LONG)((LPCREATESTRUCT)lParam)->lpCreateParams));

            //Store with the window
            SetWindowLong(hWnd, 0, (LONG)(UINT)pAV);
            SetDDECallbackPAV(pAV);
            return (DefWindowProc(hWnd, iMsg, wParam, lParam));

        case WM_DESTROY:
            PostQuitMessage(0);
            break;


        case WM_CLOSE:
            //Turn off any advise loop we have going.
            DdeClientTransaction(NULL, 0L, pAV->hConv, pAV->hszItemData
                , pAV->cfItem, XTYP_ADVSTOP, 1000L, NULL);

            //Close our connection.
            if (NULL!=pAV->hConv)
                DdeDisconnect(pAV->hConv);

            DestroyWindow(hWnd);
            break;


        case WM_PAINT:
            //Draw the current data in the client area.
            hDC=BeginPaint(hWnd, &ps);
            GetClientRect(hWnd, &rc);

            SetTextColor(hDC, GetSysColor(COLOR_WINDOWTEXT));
            SetBkColor(hDC, GetSysColor(COLOR_WINDOW));
            cch=wsprintf(pAV->szTemp, "Text:\t%s\nNumber:\t%lu"
                         , (LPSTR)pAV->xData.szText, pAV->xData.dw);

            DrawText(hDC, pAV->szTemp, cch, &rc, DT_EXPANDTABS | DT_LEFT);
            EndPaint(hWnd, &ps);
            break;



        case WM_COMMAND:
            //Handle changing the conversation complexity level.
            if (wID >=IDM_CONVERSATIONFIRST && wID <= IDM_CONVERSATIONLAST)
                {
                hMenu=GetMenu(hWnd);

                //Uncheck the current selection
                CheckMenuItem(hMenu, IDM_CONVERSATIONFIRST+pAV->iDL
                              , MF_UNCHECKED | MF_ENABLED | MF_BYCOMMAND);

                //Check the selected one and set it in pAV
                pAV->iDL=(wID-IDM_CONVERSATIONFIRST);

                CheckMenuItem(hMenu, wID, MF_CHECKED | MF_ENABLED | MF_BYCOMMAND);
                break;
                }

            switch (wID)
                {
                case IDM_NDDECONNECT:
                    if (FEstablishFirstConnection(pAV))
                        {
                        EnableConnectMenuItems(hWnd, TRUE);
                        EnableServerMenuItems(hWnd,  (ONEWAY!=pAV->iDL));
                        }

                    break;

                case IDM_NDDEDISCONNECT:
                    TerminateAllConnections(pAV);
                    EnableConnectMenuItems(hWnd, FALSE);
                    EnableServerMenuItems(hWnd,  FALSE);

                    //Also stop blasting if that's going on...
                    if (pAV->fBlasting)
                        SendMessage(hWnd, WM_COMMAND, IDM_NDDEBLASTDATASTOP, 0L);

                    break;


                case IDM_NDDECHANGETEXT:
                    /*
                     * Each time we execute this command, we'll increment
                     * each character in the text string and DdePostAdvise.
                     */
                    cch=lstrlen(pAV->xData.szText);

                    for (i=0; i<cch; i++)
                        {
                        //Restrict to printable ASCII
                        pAV->xData.szText[i]=(char)((((int)pAV->xData.szText[i]-32)%89)+33);
                        }

                    pAV->xData.wType=XDATATYPE_TEXT;

                    //Update data for all clients and poke it to server peers.
                    DataChanged(pAV);
                    break;


                case IDM_NDDECHANGENUMBER:
                    //For a number change, XOR with (invert right shift one).
                    pAV->xData.dw^=(~pAV->xData.dw) >> 1;
                    pAV->xData.wType=XDATATYPE_NUMBER;

                    //Update data for all clients and poke it to server peers.
                    DataChanged(pAV);
                    break;


                case IDM_NDDEBLASTDATASTART:
                    /*
                     * Start a timer for the minimum time (55ms) to blast
                     * data to our peer.  Motivation:  If you wanted to do
                     * some sort of smooth animation game peer-to-peer, a
                     * 16fps animation rate would be sufficient (maybe we
                     * only need 8fps).
                     */
                    pAV->fBlasting=TRUE;
                    SetTimer(hWnd, 100, 55, NULL);
                    EnableBlastMenuItems(hWnd, TRUE);
                    break;


                case IDM_NDDEBLASTDATASTOP:
                    pAV->fBlasting=FALSE;
                    KillTimer(hWnd, 100);
                    EnableBlastMenuItems(hWnd, FALSE);
                    break;


                case IDM_NDDEEXIT:
                    PostMessage(hWnd, WM_CLOSE, 0, 0L);
                    break;


                case IDM_HELPABOUT:
                    pfn=(DLGPROC)MakeProcInstance((FARPROC)AboutProc, pAV->hInst);
                    DialogBox(pAV->hInst, MAKEINTRESOURCE(IDD_ABOUT), pAV->hWnd, pfn);
                    FreeProcInstance((FARPROC)pfn);
                    break;

                }
            break;


        case WM_TIMER:
            /*
             * On every other timer we'll change the text, on the other
             * we'll change the number and advise our peer.
             */
            iMsg=(cTimers++ & 1) ? IDM_NDDECHANGETEXT : IDM_NDDECHANGENUMBER;
            SendMessage(hWnd, WM_COMMAND, iMsg, 0L);
            break;

        default:
            return (DefWindowProc(hWnd, iMsg, wParam, lParam));
        }

    return 0L;
    }






/*
 * DataChanged
 *
 * Purpose:
 *  Procedure to handle any change in our data.  It first calls DdePostAdvise
 *  to inform clients, then passes a data handle with the new data to any
 *  other server we are connected to using XTYP_POKE.  Finally, it repaints
 *  our display to show the new data.
 *
 * Parameters:
 *  pAV             PAPPVARS to the application variables.
 *
 * Return Value:
 *  None
 */

void DataChanged(PAPPVARS pAV)
    {
    HDDEDATA        hData;
    HCONV           hConv;
    int             i, cConv;

    //Update the data for peers using us as a server.
    DdePostAdvise(pAV->dwDDE, pAV->hszTopic, pAV->hszItemData);

    //Update peer servers if we're not only doing one-way
    if (ONEWAY!=pAV->iDL)
        {
        /*
         * Poke the data to peers were using as a server.  This means
         * to first create the data handle, then send it through an XTYP_POKE
         * transaction to everyone in hListC.  Creating the data handle is
         * the same code you'll find in DDE.C under XTYP_ADVREQ.
         */
        hData=DdeCreateDataHandle(pAV->dwDDE, (LPVOID)&pAV->xData, CBXDATA
                                  , 0L, pAV->hszItemData, pAV->cfItem, 0);

        cConv=(int)SendMessage(pAV->hListC, LB_GETCOUNT, 0, 0L);

        for (i=0; i < cConv; i++)
            {
            /*
             * We don't care when this finishes, and we don't even care about
             * synchronization or error handling, because this is a blind
             * transfer.
             */
            hConv=(HCONV)SendMessage(pAV->hListC, LB_GETITEMDATA, i, 0L);

            DdeClientTransaction((LPVOID)&pAV->xData, CBXDATA, hConv
                , pAV->hszItemData, CF_TEXT, XTYP_POKE
                , (DWORD)TIMEOUT_ASYNC, NULL);
            }
        }

    //Repaint our visible data as well.
    InvalidateRect(pAV->hWnd, NULL, TRUE);
    UpdateWindow(pAV->hWnd);
    return;
    }






/*
 * AboutProc
 *
 * Purpose:
 *  Dialog procedure for the ubiquitous About box.
 *
 * Parameters:
 *  Standard
 *
 * Return Value:
 *  Standard
 */

BOOL FAR PASCAL AboutProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
    {
    switch (iMsg)
        {
        case WM_INITDIALOG:
            return TRUE;

        case WM_COMMAND:
            switch (LOWORD(wParam))
                {
                case IDOK:
                    EndDialog(hDlg, TRUE);
                    break;
                }
            break;
        }

    lParam++;       //Prevent -W4 warning
    return FALSE;
    }





/*
 * EnableConnectMenuItems
 *
 * Purpose:
 *  Enables or disables/grays menu items associated with making
 *  connections, meaning Connect, Disconnect, and the conversation
 *  complexity items.
 *
 * Parameters:
 *  hWnd            HWND of the window with the menu.
 *  fConnected      BOOL TRUE if we're connected (connect disable) or
 *                  FALSE if the connect items should be enabled.
 *
 * Return Value:
 *  None
 */

void EnableConnectMenuItems(HWND hWnd, BOOL fConnected)
    {
    HMENU       hMenu;
    UINT        uStateE, uStateD;

    hMenu=GetMenu(hWnd);

    uStateE=MF_BYCOMMAND | MF_ENABLED;
    uStateD=MF_BYCOMMAND | MF_DISABLED | MF_GRAYED;

    EnableMenuItem(hMenu, IDM_NDDECONNECT,    fConnected ? uStateD : uStateE);
    EnableMenuItem(hMenu, IDM_NDDEDISCONNECT, fConnected ? uStateE : uStateD);

    EnableMenuItem(hMenu, IDM_CONVERSATIONONEWAY,   fConnected ? uStateD : uStateE);
    EnableMenuItem(hMenu, IDM_CONVERSATIONTWOWAY,   fConnected ? uStateD : uStateE);
    EnableMenuItem(hMenu, IDM_CONVERSATIONMULTIWAY, fConnected ? uStateD : uStateE);
    return;
    }




/*
 * EnableServerMenuItems
 *
 * Purpose:
 *  Enables or disables/grays menu items associated with being a
 *  server, meaning Change Text, Change Number, and Start/Stop Blasting
 *  Data.
 *
 * Parameters:
 *  hWnd            HWND of the window with the menu.
 *  fServer         BOOL TRUE if we're acting as a server (enabling the
 *                  items) or FALSE otherwise.
 *
 * Return Value:
 *  None
 */

void EnableServerMenuItems(HWND hWnd, BOOL fServer)
    {
    HMENU       hMenu;
    UINT        uStateE, uStateD;

    hMenu=GetMenu(hWnd);

    uStateE=MF_BYCOMMAND | MF_ENABLED;
    uStateD=MF_BYCOMMAND | MF_DISABLED | MF_GRAYED;

    EnableMenuItem(hMenu, IDM_NDDECHANGETEXT,     fServer ? uStateE : uStateD);
    EnableMenuItem(hMenu, IDM_NDDECHANGENUMBER,   fServer ? uStateE : uStateD);
    EnableMenuItem(hMenu, IDM_NDDEBLASTDATASTART, fServer ? uStateE : uStateD);
    EnableMenuItem(hMenu, IDM_NDDEBLASTDATASTART, fServer ? uStateE : uStateD);

    return;
    }




/*
 * EnableBlastMenuItems
 *
 * Purpose:
 *  Enables or disables/grays menu items associated with blasting
 *  data at a high speed depending on whether the option is on or off.
 *
 * Parameters:
 *  hWnd            HWND of the window with the menu.
 *  fBlasting       BOOL TRUE if we're blasting data, FALSE otherwise.
 *
 * Return Value:
 *  None
 */

void EnableBlastMenuItems(HWND hWnd, BOOL fBlast)
    {
    HMENU       hMenu;
    UINT        uStateE, uStateD;

    hMenu=GetMenu(hWnd);

    uStateE=MF_BYCOMMAND | MF_ENABLED;
    uStateD=MF_BYCOMMAND | MF_DISABLED | MF_GRAYED;

    EnableMenuItem(hMenu, IDM_NDDEBLASTDATASTART, fBlast ? uStateD : uStateE);
    EnableMenuItem(hMenu, IDM_NDDEBLASTDATASTOP,  fBlast ? uStateE : uStateD);

    return;
    }
