/*
 * EXCHANGE.H
 *
 * Definitions, structures, types, and function prototypes.
 *
 * Copyright (c)1992 Microsoft Corporation, All Right Reserved
 *
 * Kraig Brockschmidt, Software Design Engineer
 * Microsoft Systems Developer Relations
 * One Microsoft Way
 * Redmond, WA  98052
 *
 * Internet  :  kraigb@microsoft.com
 * Compuserve:  70750,2344
 * Fax       :  (206)936-7329
 */


#include "exdebug.h"

/*
 * Resource identifiers
 */

//Dialog boxes
#define IDD_ABOUT                           10
#define IDD_CONNECT                         11

//Controls
#define ID_NULL                             -1
#define ID_SERVERNAME                       100


//Menus & menu commands
#define IDR_MENU                            1


#define IDM_NDDECONNECT                     100
#define IDM_NDDEDISCONNECT                  101
#define IDM_NDDECHANGETEXT                  102
#define IDM_NDDECHANGENUMBER                103
#define IDM_NDDEBLASTDATASTART              104
#define IDM_NDDEBLASTDATASTOP               105
#define IDM_NDDEEXIT                        106

#define IDM_NDDEFIRST                       IDM_NDDECONNECT
#define IDM_NDDELAST                        IDM_NDDEEXIT

#define IDM_CONVERSATIONONEWAY              200
#define IDM_CONVERSATIONTWOWAY              201
#define IDM_CONVERSATIONMULTIWAY            202

#define IDM_CONVERSATIONFIRST               IDM_CONVERSATIONONEWAY
#define IDM_CONVERSATIONLAST                IDM_CONVERSATIONMULTIWAY


#define IDM_HELPABOUT                       300

#define IDM_HELPFIRST                       IDM_HELPABOUT
#define IDM_HELPLAST                        IDM_HELPABOUT



//Stringtable.  Keep in SEQUENTIAL order and use 0-n
#define IDS_CAPTION                         0
#define IDS_NODDEML                         1
#define IDS_DDECONNECTFAIL                  2
#define IDS_PEERCONNECTFAIL                 3
#define IDS_CONNECTIONTERM                  4
#define IDS_NOSELFCONNECT                   5
#define IDS_CLASSNAME                       6
#define IDS_SHARE                           7
#define IDS_SERVICE                         8
#define IDS_NETWORKDDE                      9
#define IDS_TOPIC                           10
#define IDS_ITEMDATA                        11
#define IDS_ITEMPEERLIST                    12
#define IDS_ITEMLOCALNAME                   13

//Number of strings and maximum string length.
#define CSTRINGS                            14
#define CCHSTRINGMAX                        80



/*
 * Application-defined types.
 */

//Data that we transfer between DDE applications.
typedef struct
    {
    WORD        wType;
    char        szText[28];
    DWORD       dw;
    } XDATA;

typedef XDATA *PXDATA;
#define CBXDATA    sizeof(XDATA)

//Data types in XDATA
#define XDATATYPE_TEXT       0
#define XDATATYPE_NUMBER     1


typedef enum
    {
    ONEWAY=0, TWOWAY, MULTI
    } DEMOLEVEL;


//Application-wide Variables
typedef struct
    {
    HINSTANCE   hInst;                  //WinMain parameters
    HINSTANCE   hInstPrev;
    LPSTR       pszCmdLine;
    int         nCmdShow;

    HWND        hWnd;                   //Main window handle
    HANDLE      hStrings;               //Handle to string memory
    char       *rgpsz[CSTRINGS];        //Array of stringtable pointers
    BOOL        fBlasting;

    //DDEML necessities initialized at startup.
    DWORD       dwDDE;                  //DDEML instance identifier
    FARPROC     pfnDDEHandler;          //DDEML Transaction Handler function
    HSZ         hszShare;               //DDEML handle to our share
    HSZ         hszService;             //DDEML handle to our service
    HSZ         hszTopic;               //DDEML handle to our topic
    HSZ         hszItemData;            //DDEML handle to our single data item
    HSZ         hszItemPeer;            //DDEML handle to a peer list
    HSZ         hszItemLocal;           //DDEML handle to our local name
    UINT        cfItem;                 //Registered format of our data

    //Other DDE data handled dynamically
    XDATA       xData;                  //Current data, coordinated with peers
    HCONV       hConv;                  //Conversation handle from DdeConnect
    char        szLocal[15];  //Our computer name
    HWND        hListC;                 //Peers we connected to
    HWND        hListS;                 //Peers that connected to us
    DEMOLEVEL   iDL;                    //How far do we do connections?

    char        szTemp[128];            //Scratch space
    char        szPeer[15];

    //Other DDE data handled dynamically
    BOOL        fFirstConnect;
    BOOL        fTimeout;
    BOOL        fXActPending;
    BOOL        fAsyncError;
    BOOL        fBlockRequest;
    } APPVARS;

typedef APPVARS *PAPPVARS;
#define CBAPPVARS sizeof(APPVARS)


//Macro to look up a string in our app's pre-loaded stringtable
#define PSZ(i)  (pAV->rgpsz[i])



/*
 * Function prototypes.
 */

//INIT.C
PAPPVARS            AppPAllocate(PINT, HINSTANCE, HINSTANCE, LPSTR, int);
BOOL                AppDDEInitialize(PAPPVARS);
PAPPVARS            AppPFree(PAPPVARS);
BOOL                AppDDEFree(PAPPVARS);
HANDLE              AppHStringCache(HINSTANCE, char **);


//EXCHANGE.C
LONG     FAR PASCAL ExchangeWndProc(HWND, UINT, WPARAM, LONG);
void                DataChanged(PAPPVARS);
void                EnableConnectMenuItems(HWND, BOOL);
void                EnableServerMenuItems(HWND, BOOL);
void                EnableBlastMenuItems(HWND, BOOL);
BOOL     FAR PASCAL AboutProc(HWND, UINT, WPARAM, LONG);


//DDE.C
void                SetDDECallbackPAV(PAPPVARS);
HDDEDATA FAR PASCAL DDETransactionHandler(UINT, UINT, HCONV, HSZ, HSZ, HDDEDATA, DWORD, DWORD);
BOOL                FDDETransactionWait(PAPPVARS);
void FAR PASCAL     TimeoutProc(HWND, UINT, UINT, DWORD);
